@@ -26,6 +26,7 @@
 -define(RABBIT_TCP_OPTS, [binary, {packet, 0}, {active,false}, {nodelay, true}]).
 -define(SOCKET_CLOSING_TIMEOUT, 1000).
 -define(HANDSHAKE_RECEIVE_TIMEOUT, 60000).
+-define(TCP_MAX_PACKET_SIZE, (16#4000000 + ?EMPTY_FRAME_SIZE - 1)).
 
 -record(state, {sock,
                 heartbeat,
@@ -41,8 +42,8 @@
 init([]) ->
     {ok, #state{}}.
 
-open_channel_args(#state{sock = Sock}) ->
-    [Sock].
+open_channel_args(#state{sock = Sock, frame_max = FrameMax}) ->
+    [Sock, FrameMax].
 
 do(#'connection.close_ok'{} = CloseOk, State) ->
     erlang:send_after(?SOCKET_CLOSING_TIMEOUT, self(), socket_closing_timeout),
@@ -51,7 +52,7 @@ do(Method, State) ->
     do2(Method, State).
 
 do2(Method, #state{writer0 = Writer}) ->
-    %% Catching because it expects the {channel_exit, _} message on error
+    %% Catching because it expects the {channel_exit, _, _} message on error
     catch rabbit_writer:send_command_sync(Writer, Method).
 
 handle_message(socket_closing_timeout,
@@ -64,7 +65,7 @@ handle_message(socket_closed, State = #state{waiting_socket_close = false}) ->
     {stop, socket_closed_unexpectedly, State};
 handle_message({socket_error, _} = SocketError, State) ->
     {stop, SocketError, State};
-handle_message({channel_exit, Reason}, State) ->
+handle_message({channel_exit, 0, Reason}, State) ->
     {stop, {channel0_died, Reason}, State};
 handle_message(heartbeat_timeout, State) ->
     {stop, heartbeat_timeout, State};
@@ -214,12 +215,21 @@ tune(#'connection.tune'{channel_max = ServerChannelMax,
                               lists:max([Client, Server]);
                           (Client, Server) ->
                               lists:min([Client, Server])
-                      end, [ClientChannelMax, ClientHeartbeat, ClientFrameMax],
-                           [ServerChannelMax, ServerHeartbeat, ServerFrameMax]),
-    NewState = State#state{heartbeat = Heartbeat, frame_max = FrameMax},
+                      end,
+                      [ClientChannelMax, ClientHeartbeat, ClientFrameMax],
+                      [ServerChannelMax, ServerHeartbeat, ServerFrameMax]),
+    %% If we attempt to recv > 64Mb, inet_drv will return enomem, so
+    %% we cap the max negotiated frame size accordingly. Note that
+    %% since we receive the frame header separately, we can actually
+    %% cope with frame sizes of 64M + ?EMPTY_FRAME_SIZE - 1.
+    CappedFrameMax = case FrameMax of
+                         0 -> ?TCP_MAX_PACKET_SIZE;
+                         _ -> lists:min([FrameMax, ?TCP_MAX_PACKET_SIZE])
+                     end,
+    NewState = State#state{heartbeat = Heartbeat, frame_max = CappedFrameMax},
     start_heartbeat(SHF, NewState),
     {#'connection.tune_ok'{channel_max = ChannelMax,
-                           frame_max   = FrameMax,
+                           frame_max   = CappedFrameMax,
                            heartbeat   = Heartbeat}, ChannelMax, NewState}.
 
 start_heartbeat(SHF, #state{sock = Sock, heartbeat = Heartbeat}) ->
@@ -299,6 +309,10 @@ handshake_recv(Expecting) ->
             end;
         {socket_error, _} = SocketError ->
             exit({SocketError, {expecting, Expecting}});
+        {refused, Version} ->
+            exit({server_refused_connection, Version});
+        {malformed_header, All} ->
+            exit({server_sent_malformed_header, All});
         heartbeat_timeout ->
             exit(heartbeat_timeout);
         Other ->
