@@ -37,6 +37,7 @@
 %%---------------------------------------------------------------------------
 
 amqp_uri_parse_test_()                  -> ?RUN([]).
+route_destination_test_()               -> ?RUN([]).
 basic_get_test_()                       -> ?RUN([]).
 basic_get_ipv6_test_()                  -> ?RUN([]).
 basic_return_test_()                    -> ?RUN([]).
@@ -61,11 +62,12 @@ async_sync_method_serialization_test_() -> ?RUN([]).
 sync_async_method_serialization_test_() -> ?RUN([]).
 teardown_test_()                        -> ?RUN([repeat]).
 rpc_test_()                             -> ?RUN([]).
+rpc_client_test_()                      -> ?RUN([]).
 pub_and_close_test_()                   -> ?RUN([]).
 channel_tune_negotiation_test_()        -> ?RUN([]).
 confirm_test_()                         -> ?RUN([]).
 confirm_barrier_test_()                 -> ?RUN([]).
-confirm_barrier_nop_test_()             -> ?RUN([]).
+confirm_select_before_wait_test_()      -> ?RUN([]).
 confirm_barrier_timeout_test_()         -> ?RUN([]).
 confirm_barrier_die_timeout_test_()     -> ?RUN([]).
 default_consumer_test_()                -> ?RUN([]).
