@@ -162,7 +162,7 @@ i(processors,      _State) -> erlang:system_info(logical_processors);
 i(disk_free_limit, _State) -> get_disk_free_limit();
 i(disk_free,       _State) -> get_disk_free();
 i(disk_free_alarm, _State) -> resource_alarm_set(disk);
-i(contexts,        _State) -> rabbit_mochiweb_contexts();
+i(contexts,        _State) -> rabbit_web_dispatch_contexts();
 i(uptime, _State) ->
     {Total, _} = erlang:statistics(wall_clock),
     Total;
@@ -178,7 +178,7 @@ i(auth_mechanisms, _State) ->
       fun (N) -> lists:member(list_to_atom(binary_to_list(N)), Mechanisms) end);
 i(applications, _State) ->
     [format_application(A) ||
-        A <- lists:keysort(1, application:which_applications(infinity))].
+        A <- lists:keysort(1, rabbit_misc:which_applications())].
 
 resource_alarm_set(Source) ->
     lists:member({{resource_limit, Source, node()},[]},
@@ -188,8 +188,8 @@ list_registry_plugins(Type) ->
     list_registry_plugins(Type, fun(_) -> true end).
 
 list_registry_plugins(Type, Fun) ->
-    [registry_plugin_enabled(Module:description(), Fun) ||
-        {_, Module} <- rabbit_registry:lookup_all(Type)].
+    [registry_plugin_enabled(set_plugin_name(Name, Module), Fun) ||
+        {Name, Module} <- rabbit_registry:lookup_all(Type)].
 
 registry_plugin_enabled(Desc, Fun) ->
     Desc ++ [{enabled, Fun(proplists:get_value(name, Desc))}].
@@ -199,30 +199,32 @@ format_application({Application, Description, Version}) ->
      {description, list_to_binary(Description)},
      {version, list_to_binary(Version)}].
 
+set_plugin_name(Name, Module) ->
+    [{name, list_to_binary(atom_to_list(Name))} |
+     proplists:delete(name, Module:description())].
+
 %%--------------------------------------------------------------------
 
 %% This is slightly icky in that we introduce knowledge of
-%% rabbit_mochiweb, which is not a dependency. But the last thing I
+%% rabbit_web_dispatch, which is not a dependency. But the last thing I
 %% want to do is create a rabbitmq_mochiweb_management_agent plugin.
-rabbit_mochiweb_contexts() ->
-    [format_context(C) || C <- rabbit_mochiweb_registry_list_all()].
+rabbit_web_dispatch_contexts() ->
+    [format_context(C) || C <- rabbit_web_dispatch_registry_list_all()].
 
 %% For similar reasons we don't declare a dependency on
 %% rabbitmq_mochiweb - so at startup there's no guarantee it will be
 %% running. So we have to catch this noproc.
-rabbit_mochiweb_registry_list_all() ->
-    case code:is_loaded(rabbit_mochiweb_registry) of
+rabbit_web_dispatch_registry_list_all() ->
+    case code:is_loaded(rabbit_web_dispatch_registry) of
         false -> [];
         _     -> try
-                     apply0(rabbit_mochiweb_registry, list_all, [])
+                     M = rabbit_web_dispatch_registry, %% Fool xref
+                     M:list_all()
                  catch exit:{noproc, _} ->
                          []
                  end
     end.
 
-%% Fool xref. Simply using apply(M, F, A) with constants is not enough.
-apply0(M, F, A) -> apply(M, F, A).
-
 format_context({Path, Description, Rest}) ->
     [{description, list_to_binary(Description)},
      {path,        list_to_binary("/" ++ Path)} |
@@ -242,7 +244,14 @@ format_mochiweb_option(_K, V) ->
 
 init([]) ->
     State = #state{fd_total = file_handle_cache:ulimit()},
-    {ok, emit_update(State)}.
+    %% If we emit an update straight away we will do so just before
+    %% the mgmt db starts up - and then have to wait ?REFRESH_RATIO
+    %% until we send another. So let's have a shorter wait in the hope
+    %% that the db will have started by the time we emit an update,
+    %% and thus shorten that little gap at startup where mgmt knows
+    %% nothing about any nodes.
+    erlang:send_after(1000, self(), emit_update),
+    {ok, State}.
 
 handle_call(_Req, _From, State) ->
     {reply, unknown_request, State}.
