@@ -188,12 +188,14 @@ function render(reqs, template, highlight) {
 }
 
 function update() {
+    replace_content('debug', '');
     clearInterval(timer);
     with_update(function(html) {
             update_navigation();
             replace_content('main', html);
             postprocess();
             postprocess_partial();
+            render_charts();
             maybe_scroll();
             reset_timer();
         });
@@ -215,10 +217,11 @@ function partial_update() {
                 throw("before/after mismatch");
             }
             for (var i = 0; i < befores.length; i++) {
-                $(befores[i]).replaceWith(afters[i]);
+                $(befores[i]).empty().append($(afters[i]).contents());
             }
             replace_content('scratch', '');
             postprocess_partial();
+            render_charts();
         });
     }
 }
@@ -345,45 +348,77 @@ function apply_state(reqs) {
     var reqs2 = {};
     for (k in reqs) {
         var req = reqs[k];
+        var options = {};
+        if (typeof(req) == "object") {
+            options = req.options;
+            req = req.path;
+        }
         var req2;
-        if (vhost_query(req) && current_vhost != '') {
+        if (options['vhost'] != undefined && current_vhost != '') {
             req2 = req + '/' + esc(current_vhost);
         }
         else {
             req2 = req;
         }
 
-        var qs = '';
-        if (req in SORT_QUERIES && current_sort != null) {
-            qs = '?sort=' + current_sort +
-                '&sort_reverse=' + current_sort_reverse;
+        var qs = [];
+        if (options['sort'] != undefined && current_sort != null) {
+            qs.push('sort=' + current_sort);
+            qs.push('sort_reverse=' + current_sort_reverse);
         }
+        if (options['ranges'] != undefined) {
+            for (i in options['ranges']) {
+                var type = options['ranges'][i];
+                var range = get_pref('chart-range-' + type).split('|');
+                var prefix;
+                if (type.substring(0, 8) == 'lengths-') {
+                    prefix = 'lengths';
+                }
+                else if (type.substring(0, 10) == 'msg-rates-') {
+                    prefix = 'msg_rates';
+                }
+                else if (type.substring(0, 11) == 'data-rates-') {
+                    prefix = 'data_rates';
+                }
+                qs.push(prefix + '_age=' + parseInt(range[0]));
+                qs.push(prefix + '_incr=' + parseInt(range[1]));
+            }
+        }
+        qs = qs.join('&');
+        if (qs != '') qs = '?' + qs;
 
         reqs2[k] = req2 + qs;
     }
     return reqs2;
 }
 
-function vhost_query(req) {
-    for (i in VHOST_QUERIES) {
-        var query = VHOST_QUERIES[i];
-        if (req.match(query)) return true;
-    }
-    return false;
-}
-
-function show_popup(type, text) {
+function show_popup(type, text, mode) {
     var cssClass = '.form-popup-' + type;
     function hide() {
-        $(cssClass).slideUp(200, function() {
+        if (mode == 'fade') {
+            $(cssClass).fadeOut(200, function() {
                 $(this).remove();
             });
+        }
+        else {
+            $(cssClass).slideUp(200, function() {
+                $(this).remove();
+            });
+        }
     }
 
     hide();
     $('h1').after(format('error-popup', {'type': type, 'text': text}));
-    $(cssClass).center().slideDown(200);
-    $(cssClass + ' span').click(hide);
+    if (mode == 'fade') {
+        $(cssClass).fadeIn(200);
+    }
+    else {
+        $(cssClass).center().slideDown(200);
+    }
+    $(cssClass + ' span').click(function () {
+        $('.popup-owner').removeClass('popup-owner');
+        hide();
+    });
 }
 
 function postprocess() {
@@ -406,7 +441,8 @@ function postprocess() {
         });
     $('#download-definitions').click(function() {
             var path = 'api/definitions?download=' +
-                esc($('#download-filename').val());
+                esc($('#download-filename').val()) +
+                '&auth=' + get_cookie('auth');
             window.location = path;
             setTimeout('app.run()');
             return false;
@@ -432,12 +468,32 @@ function postprocess() {
     $('.help').die().live('click', function() {
         help($(this).attr('id'))
     });
+    $('.rate-options').die().live('click', function() {
+        var remove = $('.popup-owner').length == 1 &&
+                     $('.popup-owner').get(0) == $(this).get(0);
+        $('.popup-owner').removeClass('popup-owner');
+        if (remove) {
+            $('.form-popup-rate-options').fadeOut(200, function() {
+                $(this).remove();
+            });
+        }
+        else {
+            $(this).addClass('popup-owner');
+            show_popup('rate-options', format('rate-options', {span: $(this)}),
+                       'fade');
+        }
+    });
     $('input, select').live('focus', function() {
         update_counter = 0; // If there's interaction, reset the counter.
     });
     $('.tag-link').click(function() {
         $('#tags').val($(this).attr('tag'));
     });
+    $('form.auto-submit select, form.auto-submit input').live('click', function(){
+        $(this).parents('form').submit();
+    });
+    $('#filter').die().live('keyup', debounce(update_filter, 500));
+    $('#truncate').die().live('keyup', debounce(update_truncate, 500));
     if (! user_administrator) {
         $('.administrator-only').remove();
     }
@@ -457,6 +513,13 @@ function postprocess_partial() {
             update();
         });
     $('.help').html('(?)');
+    // TODO remove this hack when we get rid of "updatable"
+    if ($('#filter-warning-show').length > 0) {
+        $('#filter-truncate').addClass('filter-warning');
+    }
+    else {
+        $('#filter-truncate').removeClass('filter-warning');
+    }
 }
 
 function update_multifields() {
@@ -502,6 +565,28 @@ function update_multifields() {
         });
 }
 
+function update_filter() {
+    current_filter = $(this).val();
+    var table = $(this).parents('table').first();
+    table.removeClass('filter-active');
+    if ($(this).val() != '') {
+        table.addClass('filter-active');
+    }
+    partial_update();
+}
+
+function update_truncate() {
+    var current_truncate_str =
+        $(this).val().replace(new RegExp('\\D', 'g'), '');
+    if (current_truncate_str == '')
+        current_truncate_str = '0';
+    if ($(this).val() != current_truncate_str)
+        $(this).val(current_truncate_str);
+    current_truncate = parseInt(current_truncate_str, 10);
+    store_pref('truncate', current_truncate);
+    partial_update();
+}
+
 function setup_visibility() {
     $('div.section,div.section-hidden').each(function(_index) {
         var pref = section_pref(current_template,
@@ -659,15 +744,15 @@ function sync_put(sammy, path_template) {
     return sync_req('PUT', sammy.params, path_template);
 }
 
-function sync_delete(sammy, path_template) {
-    return sync_req('DELETE', sammy.params, path_template);
+function sync_delete(sammy, path_template, options) {
+    return sync_req('DELETE', sammy.params, path_template, options);
 }
 
 function sync_post(sammy, path_template) {
     return sync_req('POST', sammy.params, path_template);
 }
 
-function sync_req(type, params0, path_template) {
+function sync_req(type, params0, path_template, options) {
     var params;
     var path;
     try {
@@ -681,6 +766,15 @@ function sync_req(type, params0, path_template) {
     req.open(type, 'api' + path, false);
     req.setRequestHeader('content-type', 'application/json');
     req.setRequestHeader('authorization', auth_header());
+
+    if (options != undefined || options != null) {
+	if (options.headers != undefined || options.headers != null) {
+	    jQuery.each(options.headers, function (k, v) {
+		req.setRequestHeader(k, v);
+	    });
+	}
+    }
+
     try {
         if (type == 'GET')
             req.send(null);
@@ -848,7 +942,8 @@ function maybe_remove_fields(params) {
     return params;
 }
 
-function put_parameter(sammy, mandatory_keys, num_keys, bool_keys) {
+function put_parameter(sammy, mandatory_keys, num_keys, bool_keys,
+                       arrayable_keys) {
     for (var i in sammy.params) {
         if (i === 'length' || !sammy.params.hasOwnProperty(i)) continue;
         if (sammy.params[i] == '' && jQuery.inArray(i, mandatory_keys) == -1) {
@@ -860,6 +955,12 @@ function put_parameter(sammy, mandatory_keys, num_keys, bool_keys) {
         else if (jQuery.inArray(i, bool_keys) != -1) {
             sammy.params[i] = sammy.params[i] == 'true';
         }
+        else if (jQuery.inArray(i, arrayable_keys) != -1) {
+            sammy.params[i] = sammy.params[i].split(' ');
+            if (sammy.params[i].length == 1) {
+                sammy.params[i] = sammy.params[i][0];
+            }
+        }
     }
     var params = {"component": sammy.params.component,
                   "vhost":     sammy.params.vhost,
@@ -937,3 +1038,19 @@ function encode_utf8(str) {
         }
     });
 })(jQuery);
+
+function debounce(f, delay) {
+    var timeout = null;
+
+    return function() {
+        var obj = this;
+        var args = arguments;
+
+        function delayed () {
+            f.apply(obj, args);
+            timeout = null;
+        }
+        if (timeout) clearTimeout(timeout);
+        timeout = setTimeout(delayed, delay);
+    }
+}
