@@ -20,6 +20,9 @@
 
 -export([http_get/1, http_put/3, http_delete/2]).
 
+-import(rabbit_mgmt_test_util, [assert_list/2, assert_item/2, test_item/2]).
+-import(rabbit_misc, [pget/2]).
+
 overview_test() ->
     %% Rather crude, but this req doesn't say much and at least this means it
     %% didn't blow up.
@@ -186,7 +189,7 @@ permissions_test() ->
 
 connections_test() ->
     {ok, Conn} = amqp_connection:start(#amqp_params_network{}),
-    LocalPort = rabbit_mgmt_test_db:local_port(Conn),
+    LocalPort = local_port(Conn),
     Path = binary_to_list(
              rabbit_mgmt_format:print(
                "/connections/127.0.0.1%3A~w%20->%20127.0.0.1%3A5672",
@@ -214,14 +217,14 @@ exchanges_test() ->
     http_put("/exchanges/myvhost/foo", Good, ?NO_CONTENT),
     http_put("/exchanges/myvhost/foo", Good, ?NO_CONTENT),
     http_get("/exchanges/%2f/foo", ?NOT_FOUND),
-    [{name,<<"foo">>},
-     {vhost,<<"myvhost">>},
-     {type,<<"direct">>},
-     {durable,true},
-     {auto_delete,false},
-     {internal,false},
-     {arguments,[]}] =
-        http_get("/exchanges/myvhost/foo"),
+    assert_item([{name,<<"foo">>},
+                 {vhost,<<"myvhost">>},
+                 {type,<<"direct">>},
+                 {durable,true},
+                 {auto_delete,false},
+                 {internal,false},
+                 {arguments,[]}],
+                http_get("/exchanges/myvhost/foo")),
 
     http_put("/exchanges/badvhost/bar", Good, ?NOT_FOUND),
     http_put("/exchanges/myvhost/bar", [{type, <<"bad_exchange_type">>}],
@@ -483,7 +486,7 @@ get_conn(Username, Password) ->
     {ok, Conn} = amqp_connection:start(#amqp_params_network{
                                         username = list_to_binary(Username),
                                         password = list_to_binary(Password)}),
-    LocalPort = rabbit_mgmt_test_db:local_port(Conn),
+    LocalPort = local_port(Conn),
     ConnPath = rabbit_misc:format(
                  "/connections/127.0.0.1%3A~w%20->%20127.0.0.1%3A5672",
                  [LocalPort]),
@@ -558,27 +561,46 @@ unicode_test() ->
     ok.
 
 defs(Key, URI, CreateMethod, Args) ->
+    defs(Key, URI, CreateMethod, Args,
+         fun(URI2) -> http_delete(URI2, ?NO_CONTENT) end).
+
+defs_v(Key, URI, CreateMethod, Args) ->
+    Rep1 = fun (S, S2) -> re:replace(S, "<vhost>", S2, [{return, list}]) end,
+    Rep2 = fun (L, V2) -> lists:keymap(fun (vhost) -> V2;
+                                           (V)     -> V end, 2, L) end,
+    %% Test against default vhost
+    defs(Key, Rep1(URI, "%2f"), CreateMethod, Rep2(Args, <<"/">>)),
+
+    %% Test against new vhost
+    http_put("/vhosts/test", [], ?NO_CONTENT),
+    PermArgs = [{configure, <<".*">>}, {write, <<".*">>}, {read, <<".*">>}],
+    http_put("/permissions/test/guest", PermArgs, ?NO_CONTENT),
+    defs(Key, Rep1(URI, "test"), CreateMethod, Rep2(Args, <<"test">>),
+         fun(URI2) -> http_delete(URI2, ?NO_CONTENT),
+                      http_delete("/vhosts/test", ?NO_CONTENT) end).
+
+defs(Key, URI, CreateMethod, Args, DeleteFun) ->
     %% Create the item
     URI2 = case CreateMethod of
                put   -> http_put(URI, Args, ?NO_CONTENT),
                         URI;
                post  -> Headers = http_post(URI, Args, ?CREATED),
-                        "../../../.." ++ Loc = pget("location", Headers),
-                        "/" ++ atom_to_list(Key) ++ Loc
+                        rabbit_web_dispatch_util:unrelativise(
+                          URI, pget("location", Headers))
            end,
     %% Make sure it ends up in definitions
     Definitions = http_get("/definitions", ?OK),
     true = lists:any(fun(I) -> test_item(Args, I) end, pget(Key, Definitions)),
 
     %% Delete it
-    http_delete(URI2, ?NO_CONTENT),
+    DeleteFun(URI2),
 
     %% Post the definitions back, it should get recreated in correct form
     http_post("/definitions", Definitions, ?CREATED),
     assert_item(Args, http_get(URI2, ?OK)),
 
     %% And delete it again
-    http_delete(URI2, ?NO_CONTENT),
+    DeleteFun(URI2),
 
     ok.
 
@@ -586,25 +608,25 @@ definitions_test() ->
     rabbit_runtime_parameters_test:register(),
     rabbit_runtime_parameters_test:register_policy_validator(),
 
-    defs(queues, "/queues/%2f/my-queue", put,
-         [{name,    <<"my-queue">>},
-          {durable, true}]),
-    defs(exchanges, "/exchanges/%2f/my-exchange", put,
-         [{name, <<"my-exchange">>},
-          {type, <<"direct">>}]),
-    defs(bindings, "/bindings/%2f/e/amq.direct/e/amq.fanout", post,
-         [{routing_key, <<"routing">>}, {arguments, []}]),
-    defs(policies, "/policies/%2f/my-policy", put,
-         [{vhost,      <<"/">>},
-          {name,       <<"my-policy">>},
-          {pattern,    <<".*">>},
-          {definition, [{testpos, [1, 2, 3]}]},
-          {priority,   1}]),
-    defs(parameters, "/parameters/test/%2f/good", put,
-         [{vhost,     <<"/">>},
-          {component, <<"test">>},
-          {name,      <<"good">>},
-          {value,     <<"ignore">>}]),
+    defs_v(queues, "/queues/<vhost>/my-queue", put,
+           [{name,    <<"my-queue">>},
+            {durable, true}]),
+    defs_v(exchanges, "/exchanges/<vhost>/my-exchange", put,
+           [{name, <<"my-exchange">>},
+            {type, <<"direct">>}]),
+    defs_v(bindings, "/bindings/<vhost>/e/amq.direct/e/amq.fanout", post,
+           [{routing_key, <<"routing">>}, {arguments, []}]),
+    defs_v(policies, "/policies/<vhost>/my-policy", put,
+           [{vhost,      vhost},
+            {name,       <<"my-policy">>},
+            {pattern,    <<".*">>},
+            {definition, [{testpos, [1, 2, 3]}]},
+            {priority,   1}]),
+    defs_v(parameters, "/parameters/test/<vhost>/good", put,
+           [{vhost,     vhost},
+            {component, <<"test">>},
+            {name,      <<"good">>},
+            {value,     <<"ignore">>}]),
     defs(users, "/users/myuser", put,
          [{name,          <<"myuser">>},
           {password_hash, <<"WAbU0ZIcvjTpxM3Q3SbJhEAM2tQ=">>},
@@ -747,6 +769,13 @@ queue_purge_test() ->
     http_delete("/queues/%2f/myqueue", ?NO_CONTENT),
     ok.
 
+queue_actions_test() ->
+    http_put("/queues/%2f/q", [], ?NO_CONTENT),
+    http_post("/queues/%2f/q/actions", [{action, sync}], ?NO_CONTENT),
+    http_post("/queues/%2f/q/actions", [{action, cancel_sync}], ?NO_CONTENT),
+    http_post("/queues/%2f/q/actions", [{action, change_colour}], ?BAD_REQUEST),
+    http_delete("/queues/%2f/q", ?NO_CONTENT),
+    ok.
 
 exclusive_consumer_test() ->
     {ok, Conn} = amqp_connection:start(#amqp_params_network{}),
@@ -808,6 +837,8 @@ columns_test() ->
              ?NO_CONTENT),
     [[{name, <<"test">>}, {arguments, [{foo, <<"bar">>}]}]] =
         http_get("/queues?columns=arguments.foo,name", ?OK),
+    [{name, <<"test">>}, {arguments, [{foo, <<"bar">>}]}] =
+        http_get("/queues/%2f/test?columns=arguments.foo,name", ?OK),
     http_delete("/queues/%2f/test", ?NO_CONTENT),
     ok.
 
@@ -951,7 +982,7 @@ parameters_test() ->
     assert_list(List, http_get("/parameters")),
     assert_list(List, http_get("/parameters/test")),
     assert_list(List, http_get("/parameters/test/%2f")),
-    http_get("/parameters/oops", ?NOT_FOUND),
+    assert_list([],   http_get("/parameters/oops")),
     http_get("/parameters/test/oops", ?NOT_FOUND),
 
     assert_item(Good,  http_get("/parameters/test/%2f/good", ?OK)),
@@ -1017,6 +1048,11 @@ msg(Key, Headers, Body, Enc) ->
      {payload,          Body},
      {payload_encoding, Enc}].
 
+local_port(Conn) ->
+    [{sock, Sock}] = amqp_connection:info(Conn, [sock]),
+    {ok, Port} = inet:port(Sock),
+    Port.
+
 %%---------------------------------------------------------------------------
 http_get(Path) ->
     http_get(Path, ?OK).
@@ -1102,35 +1138,3 @@ auth_header(Username, Password) ->
     {"Authorization",
      "Basic " ++ binary_to_list(base64:encode(Username ++ ":" ++ Password))}.
 
-%%---------------------------------------------------------------------------
-
-assert_list(Exp, Act) ->
-    case length(Exp) == length(Act) of
-        true  -> ok;
-        false -> throw({expected, Exp, actual, Act})
-    end,
-    [case length(lists:filter(fun(ActI) -> test_item(ExpI, ActI) end, Act)) of
-         1 -> ok;
-         N -> throw({found, N, ExpI, in, Act})
-     end || ExpI <- Exp].
-
-assert_item(Exp, Act) ->
-    case test_item0(Exp, Act) of
-        [] -> ok;
-        Or -> throw(Or)
-    end.
-
-test_item(Exp, Act) ->
-    case test_item0(Exp, Act) of
-        [] -> true;
-        _  -> false
-    end.
-
-test_item0(Exp, Act) ->
-    [{did_not_find, ExpI, in, Act} || ExpI <- Exp,
-                                      not lists:member(ExpI, Act)].
-
-%%---------------------------------------------------------------------------
-
-pget(K, L) ->
-     proplists:get_value(K, L).
