@@ -22,14 +22,14 @@
          code_change/3, terminate/2]).
 
 -include_lib("amqp_client/include/amqp_client.hrl").
+-include_lib("amqp_client/include/rabbit_routing_prefixes.hrl").
 -include("rabbit_stomp_frame.hrl").
 -include("rabbit_stomp.hrl").
--include("rabbit_stomp_prefixes.hrl").
 -include("rabbit_stomp_headers.hrl").
 
 -record(state, {session_id, channel, connection, subscriptions,
                 version, start_heartbeat_fun, pending_receipts,
-                config, dest_queues, reply_queues, frame_transformer,
+                config, route_state, reply_queues, frame_transformer,
                 adapter_info, send_fun, ssl_login_name}).
 
 -record(subscription, {dest_hdr, channel, ack_mode, multi_ack, description}).
@@ -67,7 +67,7 @@ init([SendFun, AdapterInfo, StartHeartbeatFun, SSLLoginName, Configuration]) ->
        start_heartbeat_fun = StartHeartbeatFun,
        pending_receipts    = undefined,
        config              = Configuration,
-       dest_queues         = sets:new(),
+       route_state         = rabbit_routing_util:init_state(),
        reply_queues        = dict:new(),
        frame_transformer   = undefined,
        adapter_info        = AdapterInfo,
@@ -410,7 +410,7 @@ tidy_canceled_subscription(ConsumerTag, #subscription{dest_hdr = DestHdr,
                                                  subscriptions = Subs}) ->
     ok = ensure_subchannel_closed(SubChannel, MainChannel),
     Subs1 = dict:erase(ConsumerTag, Subs),
-    {ok, Dest} = rabbit_stomp_util:parse_destination(DestHdr),
+    {ok, Dest} = rabbit_routing_util:parse_endpoint(DestHdr),
     maybe_delete_durable_sub(Dest, Frame, State#state{subscriptions = Subs1}).
 
 maybe_delete_durable_sub({topic, Name}, Frame,
@@ -420,8 +420,9 @@ maybe_delete_durable_sub({topic, Name}, Frame,
         true ->
             {ok, Id} = rabbit_stomp_frame:header(Frame, ?HEADER_ID),
             QName = rabbit_stomp_util:durable_subscription_queue(Name, Id),
-            amqp_channel:call(Channel, #'queue.delete'{queue  = QName,
-                                                       nowait = false}),
+            amqp_channel:call(Channel,
+                              #'queue.delete'{queue  = list_to_binary(QName),
+                                              nowait = false}),
             ok(State);
         false ->
             ok(State)
@@ -440,9 +441,19 @@ ensure_subchannel_closed(SubChannel, _MainChannel) ->
 with_destination(Command, Frame, State, Fun) ->
     case rabbit_stomp_frame:header(Frame, ?HEADER_DESTINATION) of
         {ok, DestHdr} ->
-            case rabbit_stomp_util:parse_destination(DestHdr) of
+            case rabbit_routing_util:parse_endpoint(DestHdr) of
                 {ok, Destination} ->
-                    Fun(Destination, DestHdr, Frame, State);
+                    case Fun(Destination, DestHdr, Frame, State) of
+                        {error, invalid_endpoint} ->
+                            error("Invalid destination",
+                                  "'~s' is not a valid destination for '~s'~n",
+                                  [DestHdr, Command],
+                                  State);
+                        {error, Reason} ->
+                            throw(Reason);
+                        Result ->
+                            Result
+                    end;
                 {error, {invalid_destination, Type, Content}} ->
                     error("Invalid destination",
                           "'~s' is not a valid ~p destination~n",
@@ -453,7 +464,7 @@ with_destination(Command, Frame, State, Fun) ->
                           "'~s' is not a valid destination.~n"
                           "Valid destination types are: ~s.~n",
                           [Content,
-                           string:join(rabbit_stomp_util:valid_dest_prefixes(),
+                           string:join(rabbit_routing_util:all_dest_prefixes(),
                                        ", ")], State)
             end;
         not_found ->
@@ -522,7 +533,7 @@ server_header() ->
 
 do_subscribe(Destination, DestHdr, Frame,
              State = #state{subscriptions = Subs,
-                            dest_queues   = DestQs,
+                            route_state   = RouteState,
                             connection    = Connection,
                             channel       = MainChannel}) ->
     Prefetch =
@@ -539,65 +550,75 @@ do_subscribe(Destination, DestHdr, Frame,
                                                      global         = false}),
                       Channel1
               end,
-
     {AckMode, IsMulti} = rabbit_stomp_util:ack_mode(Frame),
-
-    {ok, Queue, DestQs1} = ensure_queue(subscribe, Destination, Frame, Channel,
-                                        DestQs),
-
-    {ok, ConsumerTag, Description} = rabbit_stomp_util:consumer_tag(Frame),
-
-    amqp_channel:subscribe(Channel,
-                           #'basic.consume'{
-                             queue        = Queue,
-                             consumer_tag = ConsumerTag,
-                             no_local     = false,
-                             no_ack       = (AckMode == auto),
-                             exclusive    = false},
-                           self()),
-    ExchangeAndKey = rabbit_stomp_util:parse_routing_information(Destination),
-    ok = ensure_queue_binding(Queue, ExchangeAndKey, Channel),
-
-    ok(State#state{subscriptions =
-                       dict:store(ConsumerTag,
-                                  #subscription{dest_hdr    = DestHdr,
-                                                channel     = Channel,
-                                                ack_mode    = AckMode,
-                                                multi_ack   = IsMulti,
-                                                description = Description},
-                                  Subs),
-                   dest_queues = DestQs1}).
+    case ensure_endpoint(source, Destination, Frame, Channel, RouteState) of
+        {ok, Queue, RouteState1} ->
+            {ok, ConsumerTag, Description} =
+                rabbit_stomp_util:consumer_tag(Frame),
+            amqp_channel:subscribe(Channel,
+                                   #'basic.consume'{
+                                     queue        = Queue,
+                                     consumer_tag = ConsumerTag,
+                                     no_local     = false,
+                                     no_ack       = (AckMode == auto),
+                                     exclusive    = false},
+                                   self()),
+            ExchangeAndKey = rabbit_routing_util:parse_routing(Destination),
+            ok = rabbit_routing_util:ensure_binding(
+                   Queue, ExchangeAndKey, Channel),
+            ok(State#state{subscriptions =
+                               dict:store(
+                                 ConsumerTag,
+                                 #subscription{dest_hdr    = DestHdr,
+                                               channel     = Channel,
+                                               ack_mode    = AckMode,
+                                               multi_ack   = IsMulti,
+                                               description = Description},
+                                 Subs),
+                           route_state = RouteState1});
+        {error, _} = Err ->
+            Err
+    end.
 
 do_send(Destination, _DestHdr,
         Frame = #stomp_frame{body_iolist = BodyFragments},
-        State = #state{channel = Channel, dest_queues = DestQs}) ->
-    {ok, _Q, DestQs1} = ensure_queue(send, Destination, Frame, Channel, DestQs),
+        State = #state{channel = Channel, route_state = RouteState}) ->
 
-    {Frame1, State1} =
-        ensure_reply_to(Frame, State#state{dest_queues = DestQs1}),
+    case ensure_endpoint(dest, Destination, Frame, Channel, RouteState) of
 
-    Props = rabbit_stomp_util:message_properties(Frame1),
+        {ok, _Q, RouteState1} ->
 
-    {Exchange, RoutingKey} =
-        rabbit_stomp_util:parse_routing_information(Destination),
+            {Frame1, State1} =
+                ensure_reply_to(Frame, State#state{route_state = RouteState1}),
 
-    Method = #'basic.publish'{
-      exchange = list_to_binary(Exchange),
-      routing_key = list_to_binary(RoutingKey),
-      mandatory = false,
-      immediate = false},
+            Props = rabbit_stomp_util:message_properties(Frame1),
 
-    case transactional(Frame1) of
-        {yes, Transaction} ->
-            extend_transaction(Transaction,
-                               fun(StateN) ->
-                                       maybe_record_receipt(Frame1, StateN)
-                               end,
-                               {Method, Props, BodyFragments},
-                               State1);
-        no ->
-            ok(send_method(Method, Props, BodyFragments,
-                           maybe_record_receipt(Frame1, State1)))
+            {Exchange, RoutingKey} =
+                rabbit_routing_util:parse_routing(Destination),
+
+            Method = #'basic.publish'{
+              exchange = list_to_binary(Exchange),
+              routing_key = list_to_binary(RoutingKey),
+              mandatory = false,
+              immediate = false},
+
+            case transactional(Frame1) of
+                {yes, Transaction} ->
+                    extend_transaction(
+                      Transaction,
+                      fun(StateN) ->
+                              maybe_record_receipt(Frame1, StateN)
+                      end,
+                      {Method, Props, BodyFragments},
+                      State1);
+                no ->
+                    ok(send_method(Method, Props, BodyFragments,
+                                   maybe_record_receipt(Frame1, State1)))
+            end;
+
+        {error, _} = Err ->
+
+            Err
     end.
 
 create_ack_method(DeliveryTag, #subscription{multi_ack = IsMulti}) ->
@@ -668,18 +689,18 @@ ensure_reply_to(Frame = #stomp_frame{headers = Headers}, State) ->
         not_found ->
             {Frame, State};
         {ok, ReplyTo} ->
-            {ok, Destination} = rabbit_stomp_util:parse_destination(ReplyTo),
-            case Destination of
-                {temp_queue, TempQueueId} ->
+            {ok, Destination} = rabbit_routing_util:parse_endpoint(ReplyTo),
+            case rabbit_routing_util:dest_temp_queue(Destination) of
+                none ->
+                    {Frame, State};
+                TempQueueId ->
                     {ReplyQueue, State1} =
                         ensure_reply_queue(TempQueueId, State),
                     {Frame#stomp_frame{
                        headers = lists:keyreplace(
                                    ?HEADER_REPLY_TO, 1, Headers,
                                    {?HEADER_REPLY_TO, ReplyQueue})},
-                     State1};
-                _ ->
-                    {Frame, State}
+                     State1}
             end
     end.
 
@@ -889,68 +910,31 @@ millis_to_seconds(M) when M < 1000 -> 1;
 millis_to_seconds(M)               -> M div 1000.
 
 %%----------------------------------------------------------------------------
-%% Queue and Binding Setup
+%% Queue Setup
 %%----------------------------------------------------------------------------
 
-ensure_queue(subscribe, {exchange, _}, _Frame, Channel, DestQs) ->
-    %% Create anonymous, exclusive queue for SUBSCRIBE on /exchange destinations
-    #'queue.declare_ok'{queue = Queue} =
-        amqp_channel:call(Channel, #'queue.declare'{auto_delete = true,
-                                                    exclusive = true}),
-    {ok, Queue, DestQs};
-ensure_queue(send, {exchange, _}, _Frame, _Channel, DestQs) ->
-    %% Don't create queues on SEND for /exchange destinations
-    {ok, undefined, DestQs};
-ensure_queue(_, {queue, Name}, _Frame, Channel, DestQs) ->
-    %% Always create named queue for /queue destinations the first
-    %% time we encounter it
-    Queue = list_to_binary(Name),
-    DestQs1 = case sets:is_element(Queue, DestQs) of
-                  true  -> DestQs;
-                  false -> amqp_channel:cast(Channel,
-                                             #'queue.declare'{durable = true,
-                                                              queue   = Queue,
-                                                              nowait  = true}),
-                           sets:add_element(Queue, DestQs)
-              end,
-    {ok, Queue, DestQs1};
-ensure_queue(subscribe, {topic, Name}, Frame, Channel, DestQs) ->
-    %% Create queue for SUBSCRIBE on /topic destinations. Queues are
-    %% anonymous, auto_delete and exclusive for transient
-    %% subscriptions. Durable subscriptions get shared, named, durable
-    %% queues.
-    Method =
-        case rabbit_stomp_frame:boolean_header(Frame,
-                                               ?HEADER_PERSISTENT, false) of
-            true  ->
-                {ok, Id} = rabbit_stomp_frame:header(Frame, ?HEADER_ID),
-                QName = rabbit_stomp_util:durable_subscription_queue(Name, Id),
-                #'queue.declare'{durable = true, queue = QName};
+ensure_endpoint(source, EndPoint, Frame, Channel, State) ->
+    Params =
+        case rabbit_stomp_frame:boolean_header(
+               Frame, ?HEADER_PERSISTENT, false) of
+            true ->
+                [{subscription_queue_name_gen,
+                    fun () ->
+                        {ok, Id} = rabbit_stomp_frame:header(Frame, ?HEADER_ID),
+                        {_, Name} = rabbit_routing_util:parse_routing(EndPoint),
+                        list_to_binary(
+                          rabbit_stomp_util:durable_subscription_queue(Name,
+                                                                       Id))
+                    end},
+                 {durable, true}];
             false ->
-                #'queue.declare'{auto_delete = true, exclusive = true}
+                [{durable, false}]
         end,
-    #'queue.declare_ok'{queue = Queue} = amqp_channel:call(Channel, Method),
-    {ok, Queue, DestQs};
-ensure_queue(send, {topic, _}, _Frame, _Channel, DestQs) ->
-    %% Don't create queues on SEND for /topic destinations
-    {ok, undefined, DestQs};
-ensure_queue(_, {Type, Name}, _Frame, _Channel, DestQs)
-  when Type =:= reply_queue orelse Type =:= amqqueue ->
-    {ok, list_to_binary(Name), DestQs}.
+    rabbit_routing_util:ensure_endpoint(source, Channel, EndPoint, Params, State);
+
+ensure_endpoint(Direction, Endpoint, _Frame, Channel, State) ->
+    rabbit_routing_util:ensure_endpoint(Direction, Channel, Endpoint, State).
 
-ensure_queue_binding(QueueBin, {"", Queue}, _Channel) ->
-    %% i.e., we should only be asked to bind to the default exchange a
-    %% queue with its own name
-    QueueBin = list_to_binary(Queue),
-    ok;
-ensure_queue_binding(Queue, {Exchange, RoutingKey}, Channel) ->
-    #'queue.bind_ok'{} =
-        amqp_channel:call(Channel,
-                          #'queue.bind'{
-                            queue       = Queue,
-                            exchange    = list_to_binary(Exchange),
-                            routing_key = list_to_binary(RoutingKey)}),
-    ok.
 %%----------------------------------------------------------------------------
 %% Success/error handling
 %%----------------------------------------------------------------------------
