@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is VMware, Inc.
-%% Copyright (c) 2007-2012 VMware, Inc.  All rights reserved.
+%% Copyright (c) 2007-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_binary_parser).
@@ -50,47 +50,36 @@ parse_array(<<ValueAndRest/binary>>) ->
     {Type, Value, Rest} = parse_field_value(ValueAndRest),
     [{Type, Value} | parse_array(Rest)].
 
-parse_field_value(<<"S", VLen:32/unsigned, ValueString:VLen/binary, Rest/binary>>) ->
-    {longstr, ValueString, Rest};
+parse_field_value(<<"S", VLen:32/unsigned, V:VLen/binary, R/binary>>) ->
+    {longstr, V, R};
 
-parse_field_value(<<"I", Value:32/signed, Rest/binary>>) ->
-    {signedint, Value, Rest};
+parse_field_value(<<"I", V:32/signed, R/binary>>) ->
+    {signedint, V, R};
 
-parse_field_value(<<"D", Before:8/unsigned, After:32/unsigned, Rest/binary>>) ->
-    {decimal, {Before, After}, Rest};
+parse_field_value(<<"D", Before:8/unsigned, After:32/unsigned, R/binary>>) ->
+    {decimal, {Before, After}, R};
 
-parse_field_value(<<"T", Value:64/unsigned, Rest/binary>>) ->
-    {timestamp, Value, Rest};
+parse_field_value(<<"T", V:64/unsigned, R/binary>>) ->
+    {timestamp, V, R};
 
-parse_field_value(<<"F", VLen:32/unsigned, Table:VLen/binary, Rest/binary>>) ->
-    {table, parse_table(Table), Rest};
+parse_field_value(<<"F", VLen:32/unsigned, Table:VLen/binary, R/binary>>) ->
+    {table, parse_table(Table), R};
 
-parse_field_value(<<"A", VLen:32/unsigned, Array:VLen/binary, Rest/binary>>) ->
-    {array, parse_array(Array), Rest};
+parse_field_value(<<"A", VLen:32/unsigned, Array:VLen/binary, R/binary>>) ->
+    {array, parse_array(Array), R};
 
-parse_field_value(<<"b", Value:8/unsigned, Rest/binary>>) ->
-    {byte, Value, Rest};
+parse_field_value(<<"b", V:8/unsigned, R/binary>>) -> {byte,        V, R};
+parse_field_value(<<"d", V:64/float,   R/binary>>) -> {double,      V, R};
+parse_field_value(<<"f", V:32/float,   R/binary>>) -> {float,       V, R};
+parse_field_value(<<"l", V:64/signed,  R/binary>>) -> {long,        V, R};
+parse_field_value(<<"s", V:16/signed,  R/binary>>) -> {short,       V, R};
+parse_field_value(<<"t", V:8/unsigned, R/binary>>) -> {bool, (V /= 0), R};
 
-parse_field_value(<<"d", Value:64/float, Rest/binary>>) ->
-    {double, Value, Rest};
+parse_field_value(<<"x", VLen:32/unsigned, V:VLen/binary, R/binary>>) ->
+    {binary, V, R};
 
-parse_field_value(<<"f", Value:32/float, Rest/binary>>) ->
-    {float, Value, Rest};
-
-parse_field_value(<<"l", Value:64/signed, Rest/binary>>) ->
-    {long, Value, Rest};
-
-parse_field_value(<<"s", Value:16/signed, Rest/binary>>) ->
-    {short, Value, Rest};
-
-parse_field_value(<<"t", Value:8/unsigned, Rest/binary>>) ->
-    {bool, (Value /= 0), Rest};
-
-parse_field_value(<<"x", VLen:32/unsigned, ValueString:VLen/binary, Rest/binary>>) ->
-    {binary, ValueString, Rest};
-
-parse_field_value(<<"V", Rest/binary>>) ->
-    {void, undefined, Rest}.
+parse_field_value(<<"V", R/binary>>) ->
+    {void, undefined, R}.
 
 ensure_content_decoded(Content = #content{properties = Props})
   when Props =/= none ->
