@@ -22,7 +22,7 @@
          is_mirrored/1, update_mirrors/2, validate_policy/1]).
 
 %% for testing only
--export([suggested_queue_nodes/4]).
+-export([module/1]).
 
 -include("rabbit.hrl").
 
@@ -32,6 +32,8 @@
                            [policy_validator, <<"ha-mode">>, ?MODULE]}},
                     {mfa, {rabbit_registry, register,
                            [policy_validator, <<"ha-params">>, ?MODULE]}},
+                    {mfa, {rabbit_registry, register,
+                           [policy_validator, <<"ha-sync-mode">>, ?MODULE]}},
                     {requires, rabbit_registry},
                     {enables, recovery}]}).
 
@@ -184,6 +186,7 @@ start_child(Name, MirrorNode, Q) ->
                    rabbit_mirror_queue_slave_sup:start_child(MirrorNode, [Q])
            end) of
         {ok, SPid} when is_pid(SPid)  ->
+            maybe_auto_sync(Q),
             rabbit_log:info("Adding mirror of ~s on node ~p: ~p~n",
                             [rabbit_misc:rs(Name), MirrorNode, SPid]),
             {ok, started};
@@ -214,6 +217,8 @@ report_deaths(MirrorPid, IsMaster, QueueName, DeadPids) ->
 
 store_updated_slaves(Q = #amqqueue{slave_pids      = SPids,
                                    sync_slave_pids = SSPids}) ->
+    %% TODO now that we clear sync_slave_pids in rabbit_durable_queue,
+    %% do we still need this filtering?
     SSPids1 = [SSPid || SSPid <- SSPids, lists:member(SSPid, SPids)],
     Q1 = Q#amqqueue{sync_slave_pids = SSPids1},
     ok = rabbit_amqqueue:store_queue(Q1),
@@ -234,14 +239,17 @@ suggested_queue_nodes(Q) ->
 %% This variant exists so we can pull a call to
 %% rabbit_mnesia:cluster_nodes(running) out of a loop or
 %% transaction or both.
-suggested_queue_nodes(Q, PossibleNodes) ->
-    {MNode0, SNodes} = actual_queue_nodes(Q),
+suggested_queue_nodes(Q, All) ->
+    {MNode0, SNodes, SSNodes} = actual_queue_nodes(Q),
     MNode = case MNode0 of
                 none -> node();
                 _    -> MNode0
             end,
-    suggested_queue_nodes(policy(<<"ha-mode">>, Q), policy(<<"ha-params">>, Q),
-                          {MNode, SNodes}, PossibleNodes).
+    Params = policy(<<"ha-params">>, Q),
+    case module(Q) of
+        {ok, M} -> M:suggested_queue_nodes(Params, MNode, SNodes, SSNodes, All);
+        _       -> {MNode, []}
+    end.
 
 policy(Policy, Q) ->
     case rabbit_policy:get(Policy, Q) of
@@ -249,57 +257,42 @@ policy(Policy, Q) ->
         _       -> none
     end.
 
-suggested_queue_nodes(<<"all">>, _Params, {MNode, _SNodes}, Possible) ->
-    {MNode, Possible -- [MNode]};
-suggested_queue_nodes(<<"nodes">>, Nodes0, {MNode, _SNodes}, Possible) ->
-    Nodes1 = [list_to_atom(binary_to_list(Node)) || Node <- Nodes0],
-    %% If the current master is currently not in the nodes specified,
-    %% act like it is for the purposes below - otherwise we will not
-    %% return it in the results...
-    Nodes = lists:usort([MNode | Nodes1]),
-    Unavailable = Nodes -- Possible,
-    Available = Nodes -- Unavailable,
-    case Available of
-        [] -> %% We have never heard of anything? Not much we can do but
-              %% keep the master alive.
-              {MNode, []};
-        _  -> case lists:member(MNode, Available) of
-                  true  -> {MNode, Available -- [MNode]};
-                  false -> promote_slave(Available)
-              end
+module(#amqqueue{} = Q) ->
+    case rabbit_policy:get(<<"ha-mode">>, Q) of
+        {ok, Mode} -> module(Mode);
+        _          -> not_mirrored
     end;
-%% When we need to add nodes, we randomise our candidate list as a
-%% crude form of load-balancing. TODO it would also be nice to
-%% randomise the list of ones to remove when we have too many - but
-%% that would fail to take account of synchronisation...
-suggested_queue_nodes(<<"exactly">>, Count, {MNode, SNodes}, Possible) ->
-    SCount = Count - 1,
-    {MNode, case SCount > length(SNodes) of
-                true  -> Cand = shuffle((Possible -- [MNode]) -- SNodes),
-                         SNodes ++ lists:sublist(Cand, SCount - length(SNodes));
-                false -> lists:sublist(SNodes, SCount)
-            end};
-suggested_queue_nodes(_, _, {MNode, _}, _) ->
-    {MNode, []}.
 
-shuffle(L) ->
-    {A1,A2,A3} = now(),
-    random:seed(A1, A2, A3),
-    {_, L1} = lists:unzip(lists:keysort(1, [{random:uniform(), N} || N <- L])),
-    L1.
+module(Mode) when is_binary(Mode) ->
+    case rabbit_registry:binary_to_type(Mode) of
+        {error, not_found} -> not_mirrored;
+        T                  -> case rabbit_registry:lookup_module(ha_mode, T) of
+                                  {ok, Module} -> {ok, Module};
+                                  _            -> not_mirrored
+                              end
+    end.
 
-actual_queue_nodes(#amqqueue{pid = MPid, slave_pids = SPids}) ->
+is_mirrored(Q) ->
+    case module(Q) of
+        {ok, _}  -> true;
+        _        -> false
+    end.
+
+actual_queue_nodes(#amqqueue{pid             = MPid,
+                             slave_pids      = SPids,
+                             sync_slave_pids = SSPids}) ->
+    Nodes = fun (L) -> [node(Pid) || Pid <- L] end,
     {case MPid of
          none -> none;
          _    -> node(MPid)
-     end, [node(Pid) || Pid <- SPids]}.
+     end, Nodes(SPids), Nodes(SSPids)}.
 
-is_mirrored(Q) ->
-    case policy(<<"ha-mode">>, Q) of
-        <<"all">>     -> true;
-        <<"nodes">>   -> true;
-        <<"exactly">> -> true;
-        _             -> false
+maybe_auto_sync(Q = #amqqueue{pid = QPid}) ->
+    case policy(<<"ha-sync-mode">>, Q) of
+        <<"automatic">> ->
+            spawn(fun() -> rabbit_amqqueue:sync_mirrors(QPid) end);
+        _ ->
+            ok
     end.
 
 update_mirrors(OldQ = #amqqueue{pid = QPid},
@@ -313,41 +306,42 @@ update_mirrors(OldQ = #amqqueue{pid = QPid},
 
 update_mirrors0(OldQ = #amqqueue{name = QName},
                 NewQ = #amqqueue{name = QName}) ->
-    All = fun ({A,B}) -> [A|B] end,
-    OldNodes = All(actual_queue_nodes(OldQ)),
-    NewNodes = All(suggested_queue_nodes(NewQ)),
-    add_mirrors(QName, NewNodes -- OldNodes),
+    {OldMNode, OldSNodes, _} = actual_queue_nodes(OldQ),
+    {NewMNode, NewSNodes}    = suggested_queue_nodes(NewQ),
+    OldNodes = [OldMNode | OldSNodes],
+    NewNodes = [NewMNode | NewSNodes],
+    add_mirrors (QName, NewNodes -- OldNodes),
     drop_mirrors(QName, OldNodes -- NewNodes),
+    maybe_auto_sync(NewQ),
     ok.
 
 %%----------------------------------------------------------------------------
 
 validate_policy(KeyList) ->
-    validate_policy(
-      proplists:get_value(<<"ha-mode">>,   KeyList),
-      proplists:get_value(<<"ha-params">>, KeyList, none)).
-
-validate_policy(<<"all">>, none) ->
-    ok;
-validate_policy(<<"all">>, _Params) ->
-    {error, "ha-mode=\"all\" does not take parameters", []};
-
-validate_policy(<<"nodes">>, []) ->
-    {error, "ha-mode=\"nodes\" list must be non-empty", []};
-validate_policy(<<"nodes">>, Nodes) when is_list(Nodes) ->
-    case [I || I <- Nodes, not is_binary(I)] of
-        []      -> ok;
-        Invalid -> {error, "ha-mode=\"nodes\" takes a list of strings, "
-                    "~p was not a string", [Invalid]}
-    end;
-validate_policy(<<"nodes">>, Params) ->
-    {error, "ha-mode=\"nodes\" takes a list, ~p given", [Params]};
-
-validate_policy(<<"exactly">>, N) when is_integer(N) andalso N > 0 ->
-    ok;
-validate_policy(<<"exactly">>, Params) ->
-    {error, "ha-mode=\"exactly\" takes an integer, ~p given", [Params]};
-
-validate_policy(Mode, _Params) ->
-    {error, "~p is not a valid ha-mode value", [Mode]}.
+    Mode = proplists:get_value(<<"ha-mode">>, KeyList, none),
+    Params = proplists:get_value(<<"ha-params">>, KeyList, none),
+    SyncMode = proplists:get_value(<<"ha-sync-mode">>, KeyList, none),
+    case {Mode, Params, SyncMode} of
+        {none, none, none} ->
+            ok;
+        {none, _, _} ->
+            {error, "ha-mode must be specified to specify ha-params or "
+             "ha-sync-mode", []};
+        _ ->
+            case module(Mode) of
+                {ok, M} -> case M:validate_policy(Params) of
+                               ok -> validate_sync_mode(SyncMode);
+                               E  -> E
+                           end;
+                _       -> {error, "~p is not a valid ha-mode value", [Mode]}
+            end
+    end.
 
+validate_sync_mode(SyncMode) ->
+    case SyncMode of
+        <<"automatic">> -> ok;
+        <<"manual">>    -> ok;
+        none            -> ok;
+        Mode            -> {error, "ha-sync-mode must be \"manual\" "
+                            "or \"automatic\", got ~p", [Mode]}
+    end.
