@@ -11,14 +11,14 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is VMware, Inc.
-%% Copyright (c) 2010-2012 VMware, Inc.  All rights reserved.
+%% Copyright (c) 2010-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mirror_queue_slave_sup).
 
 -behaviour(supervisor2).
 
--export([start/0, start_link/0, start_child/2]).
+-export([start_link/0, start_child/2]).
 
 -export([init/1]).
 
@@ -26,20 +26,9 @@
 
 -define(SERVER, ?MODULE).
 
-start() ->
-    {ok, _} =
-        supervisor2:start_child(
-          rabbit_sup,
-          {rabbit_mirror_queue_slave_sup,
-           {rabbit_mirror_queue_slave_sup, start_link, []},
-           transient, infinity, supervisor, [rabbit_mirror_queue_slave_sup]}),
-    ok.
+start_link() -> supervisor2:start_link({local, ?SERVER}, ?MODULE, []).
 
-start_link() ->
-    supervisor2:start_link({local, ?SERVER}, ?MODULE, []).
-
-start_child(Node, Args) ->
-    supervisor2:start_child({?SERVER, Node}, Args).
+start_child(Node, Args) -> supervisor2:start_child({?SERVER, Node}, Args).
 
 init([]) ->
     {ok, {{simple_one_for_one_terminate, 10, 10},
