@@ -47,7 +47,7 @@ setup() ->
 active() ->
     {ok, ExpandDir} = application:get_env(rabbit, plugins_expand_dir),
     InstalledPlugins = [ P#plugin.name || P <- list(ExpandDir) ],
-    [App || {App, _, _} <- application:which_applications(),
+    [App || {App, _, _} <- rabbit_misc:which_applications(),
             lists:member(App, InstalledPlugins)].
 
 %% @doc Get the list of plugins which are ready to be enabled.
@@ -64,8 +64,8 @@ list(PluginsDir) ->
                     [plugin_info(PluginsDir, Plug) || Plug <- EZs ++ FreeApps]),
     case Problems of
         [] -> ok;
-        _  -> io:format("Warning: Problem reading some plugins: ~p~n",
-                        [Problems])
+        _  -> error_logger:warning_msg(
+                "Problem reading some plugins: ~p~n", [Problems])
     end,
     Plugins.
 
@@ -112,8 +112,9 @@ prepare_plugins(EnabledFile, PluginsDistDir, ExpandDir) ->
 
     case Enabled -- plugin_names(ToUnpackPlugins) of
         []      -> ok;
-        Missing -> io:format("Warning: the following enabled plugins were "
-                             "not found: ~p~n", [Missing])
+        Missing -> error_logger:warning_msg(
+                     "The following enabled plugins were not found: ~p~n",
+                     [Missing])
     end,
 
     %% Eliminate the contents of the destination directory
