@@ -23,8 +23,6 @@
 
 -callback validate(rabbit_types:vhost(), binary(), binary(),
                    term()) -> validate_results().
--callback validate_clear(rabbit_types:vhost(), binary(),
-                         binary()) -> validate_results().
 -callback notify(rabbit_types:vhost(), binary(), binary(), term()) -> 'ok'.
 -callback notify_clear(rabbit_types:vhost(), binary(), binary()) -> 'ok'.
 
@@ -35,7 +33,6 @@
 behaviour_info(callbacks) ->
     [
      {validate, 4},
-     {validate_clear, 3},
      {notify, 4},
      {notify_clear, 3}
     ];
